// -*- C++ -*-
// main handler for TMDs
#include "TMDlib.h"
#include "TMDlibPDFpath.h"

using namespace TMDlib;


//extern int iset ;
//int imode, irep;
//extern bool first;
//bool first = true;
/*!
 * TMDinit 1st part 
 */
/*
void TMD::TMDinit(const string name, int irep, int imode ) {


    TMDinfo(name) ;
    
    iset = TMD::TMDnumberPDF(name) ;
    cout << " TMDinit 1: name = " << name << " iset = "<< iset << " irep = " << irep << " imode = " << imode << endl; 
    first=true;
    ncallTMDdensity = 0;
}
*/
/*!
 * TMDinit 2nd part 
 */
void TMD::TMDinit(const string name, int irep ) {


    TMDinfo(name) ;
    // check whether irep is within allowed range
    if (irep <= TMD::TMDgetNumMembers() ) { 
       iset = TMD::TMDnumberPDF(name) + irep ;
    
       cout << " TMDinit 2: name = " << name << " iset = " << iset << endl;
       cout << " TMDinit 2: irep = " << irep << " out of " << TMD::TMDgetNumMembers() << endl; 
       } else {
       cout << " Error: TMDinit 2: name = " << name << " irep = " << irep << " not found "<< endl; 
       cout << " Error: TMDinit 2: available Nr of sets = " << TMD::TMDgetNumMembers() << endl; 
       }
    first=true;
    ncallTMDdensity = 0;
}
/*!
 * TMDinit 3rd part 
 */
void TMD::TMDinit(const string name) {


    TMDinfo(name) ;
    
    iset = TMD::TMDnumberPDF(name) ;
    irep = 0 ;
    imode = 0;
    cout << " TMDinit 3: name = " << name <<endl; 
   // cout << " TMDinit 3: pdfpath = " << pdfpath << endl;
    first=true;
    ncallTMDdensity = 0;
}

/*!
 * TMDinit 4th part 
 */

void TMD::TMDinit(int & isetInput) {
    //cout << " TMDinit iset " << isetInput << " test fmod " << fmod(isetInput, 100)  << endl;
    // extract base name for number
    int irep = fmod(isetInput, 100);
    int isetBase = isetInput - irep ;
    //cout << " TMDinit isetBase " << isetBase << endl;
    string name = TMD::TMDstringPDF(isetBase) ;   
    TMDinfo(name) ;
    int ibase=TMD::TMDnumberPDF(name);
    iset = 0 ;
    imode = 0;
    
    if ( isetInput <= ibase + TMD::TMDgetNumMembers() ) {
       iset = isetInput ;
    
       if ( irep <= TMD::TMDgetNumMembers() ) {
           cout << " TMDinit 4: iset = "<< iset << " name " << name << " irep = " << irep <<endl; }
       }
       else {
       cout << " Error: TMDinit 4: iset = "<< iset << " name " << name << " isetInput = " << isetInput <<endl; 
       cout << " Error: TMDinit 4: available Nr of sets = " << TMD::TMDgetNumMembers() << endl; 
       }
    first=true;
    ncallTMDdensity = 0;
}

/*!
 * TMDpdf 1st
 */
vector<double> TMD::TMDpdf( double x, double xbar, double kt, double mu) {
    double top,bottom,charm,strange,uval,dval,gluon, phot ;
    double tbar, ubar, dbar, sbar, cbar, bbar, up, dn, sea;
    //Initialize output
    
      uval   = 0.;
      dval   = 0.;
      strange = 0.;
      sea = 0. ;
      charm  = 0.;
      bottom = 0.;
      top = 0; 
      gluon  = 0.;
      dbar = 0;
      ubar = 0;
      sbar = 0;
      cbar = 0;
      bbar = 0;
      vector<double> xpq(13);   
      for (int j=0; j<=12; j++){
      xpq[j] =0;
      }
      //cout<< " TMDpdf1 x:" << TMDgetXmin()<< " " << TMDgetXmax() << endl;
      //cout<< " TMDpdf1 q2:" << TMDgetQ2min()<< " " << TMDgetQ2max() << endl;
      if ( x < TMDgetXmin() || x > TMDgetXmax() )  { 
         cout<< " TMDpdf1 x:" << TMDgetXmin()<< " " << TMDgetXmax() << endl;
         return xpq;
         }
      if ( mu < sqrt(TMDgetQ2min()) || mu > sqrt(TMDgetQ2max()) ) {
         cout<< " TMDpdf1 q2 = :"<< mu<<" "  << sqrt(TMDgetQ2min())<< " " << sqrt(TMDgetQ2max()) << endl;
         return xpq;
         }
      // cout << " TMDpdf 1st " << endl;
    int kf =2212 ; /* kf = 2212 for proton */
    
    if ( x <= TMDgetXmin() ) x=TMDgetXmin();
    if ( x >= TMDgetXmax() ) x=TMDgetXmax();
    if ( mu <= sqrt(TMDgetQ2min()) ) mu=sqrt(TMDgetQ2min());
    if ( mu >= sqrt(TMDgetQ2max()) ) mu=sqrt(TMDgetQ2max());
    
    // cout << " calling TMDdensity from TMDpdf1 " << endl;
    TMDdensity( x, xbar,  kt,  mu,  up,  ubar, dn, dbar, strange, sbar, charm, cbar, bottom,bbar, gluon);
    
    uval = up-ubar ;
    dval = dn-dbar ;
    
    xpq[0] = tbar ;  
    xpq[1] = bbar ;  
    xpq[2] = cbar ;  
    xpq[3] = sbar ;  
    xpq[4] = ubar ;  
    xpq[5] = dbar ;  
    xpq[6] = gluon ;  
    xpq[7] = dval+dbar ;  
    xpq[8] = uval+ubar ;  
    xpq[9] = strange ;  
    xpq[10] = charm ;  
    xpq[11] = bottom ;  
    xpq[12] = top ;  
    return xpq ;
}


/*!
 * TMDpdf 3rd
 */
void TMD::TMDpdf( double x, double xbar, double kt, double mu, vector<double>& xpq) {
    double top,bottom,charm,sea,uval,dval,gluon,strange, phot ;
    double up, dn, dbar, ubar, sbar, cbar, bbar, tbar;


    //Initialize output
      uval   = 0.;
      ubar   = 0.;
      dval   = 0.;
      dbar   = 0.;
      strange= 0.;
      sbar   = 0.;
      charm  = 0.;
      cbar   = 0.;
      bottom = 0.;
      bbar   = 0.;
      top = 0; 
      tbar = 0.;
      gluon  = 0.;
      for (int j=0; j<=12; j++){
      xpq[j] =0;
      }
      if ( x < TMDgetXmin() || x > TMDgetXmax() )  { 
         cout<< " TMDpdf3 x:" << TMDgetXmin()<< " " << TMDgetXmax() << endl;
         return ;
         }
      if ( mu < sqrt(TMDgetQ2min()) || mu > sqrt(TMDgetQ2max()) ) {
         cout<< " TMDpdf3 q2 = :"<< mu<<" "  << sqrt(TMDgetQ2min())<< " " << sqrt(TMDgetQ2max()) << endl;
         return ;
         }

      // cout << " calling TMDdensity from TMDpdf3 " << endl;
      TMDdensity( x, xbar,  kt,  mu,  up,  ubar, dn, dbar, strange, sbar, charm, cbar, bottom,bbar, gluon);


      uval = up-ubar ;
      dval = dn-dbar ;
  
      xpq[0] = tbar ;  
      xpq[1] = bbar ;  
      xpq[2] = cbar ;  
      xpq[3] = sbar;  
      xpq[4] = ubar ;  
      xpq[5] = dbar ;  
      xpq[6] = gluon ;  
      xpq[7] = dval+dbar ;  
      xpq[8] = uval+ubar ;  
      xpq[9] = strange ;  
      xpq[10] = charm ;  
      xpq[11] = bottom ;  
      xpq[12] = top ;  
}


/*!
 * TMDpdf 5th
 */
void TMD::TMDpdf( double x, double xbar, double kt, double mu, double& uval, double& dval, double& sea, double& charm, double& bottom, double& gluon) {

      double dbar, ubar, strange, sbar, cbar, bbar, phot, top, tbar ;
      double up, dn;
      uval   = 0.;
      dval   = 0.;
      charm  = 0.;
      bottom = 0.;
      gluon  = 0.;
      // cout << " calling TMDdensity from TMDpdf5 " << endl;
      if ( x < TMDgetXmin() || x > TMDgetXmax() )  { 
         cout<< " TMDpdf5 x:" << TMDgetXmin()<< " " << TMDgetXmax() << endl;
         return ;
         }
      if ( mu < sqrt(TMDgetQ2min()) || mu > sqrt(TMDgetQ2max()) ) {
         cout<< " TMDpdf5 q2 = :"<< mu<<" "  << sqrt(TMDgetQ2min())<< " " << sqrt(TMDgetQ2max()) << endl;
         return ;
         }
      TMDdensity( x, xbar,  kt,  mu,  up,  ubar, dn, dbar, strange, sbar, charm, cbar, bottom,bbar, gluon);

      uval = up-ubar ;
      dval = dn-dbar ;
      sea = sbar ;

}

/*!
 * TMDpdf 7th
 */
void TMD::TMDpdf( double x, double xbar, double kt, double mu, double& up, double& ubar, double& dn, double& dbar, double& strange, double& sbar, double& charm,double& cbar, double& bottom,double& bbar, double& gluon) {
    // cout << " calling TMDdensity from TMDpdf7 " << iset << endl;
    TMDdensity( x, xbar,  kt,  mu,  up,  ubar, dn, dbar, strange, sbar, charm, cbar, bottom,bbar, gluon);
    }

/*!
 *   Main routine for TMDdensities
 */
void TMD::TMDdensity( double x, double xbar, double kt, double mu, double& up, double& ubar, double& dn, double& dbar, double& strange, double& sbar, double& charm,double& cbar, double& bottom,double& bbar, double& gluon) {

    //Initialize output
      double uval, dval, top,tbar,phot, sea;
      uval   = 0.;
      ubar   = 0.;
      dval   = 0.;
      dbar   = 0.;
      strange= 0.;
      sea    = 0.;
      sbar   = 0.;
      charm  = 0.;
      cbar   = 0.;
      bottom = 0.;
      bbar   = 0.;
      gluon  = 0.;
      //cout<< " TMDdensity x:" << TMDgetXmin()<< " " << TMDgetXmin() << endl;
      //cout<< " TMDdensity q:" << TMDgetQmin()<< " " << TMDgetQmin() << endl;
      if ( x < TMDgetXmin() || x > TMDgetXmax() )  { 
         ncallTMDdensity=ncallTMDdensity + 1 ;
         if (TMDgetExtrapolation_kt() == "fixed" ) { 
             if ( x < TMDgetXmin() ) 
               { x = TMDgetXmin(); }
               else {x = TMDgetXmax();} 
             if (ncallTMDdensity <=10 ) {cout<< " TMDdensity x: fixed extraploation used "  << endl;}
         }
         else{ 
         if (ncallTMDdensity <=10 ) {
             cout<< " TMDdensity x: no extraploation requested -> return 0 " << TMDgetXmin()<< " " << TMDgetXmax() << endl;}
         return ; }
         }
      if ( mu < TMDgetQmin() || mu > TMDgetQmax() ) {
         ncallTMDdensity=ncallTMDdensity + 1 ;
         if (TMDgetExtrapolation_Q2() == "fixed" ) { 
             if ( mu < TMDgetQmin() ) 
               { mu = TMDgetQmin(); }
               else {mu = TMDgetQmax();} 
             if (ncallTMDdensity <=10 ) {cout<< " TMDdensity q2: fixed extraploation used " << mu<< " " <<TMDgetQmin() <<" " << TMDgetQmax()  << endl;}
         }
         else{ 
         if (ncallTMDdensity <=10 ) {
             cout<< " TMDdensity q2: no extraploation requested -> return 0 "<< mu <<" "  << sqrt(TMDgetQ2min())<< " " << sqrt(TMDgetQ2max()) << endl;}
         return ;}
         }
    //cout << " checking on format " << NewFormat << endl;
    if (NewFormat == 1 ) {
       cout << "TMDdensity: TMDscheme = " << TMDgetScheme() << endl;
       if (  TMDgetScheme() == "PB TMD" ) { 
          cout << " TMDdensity:  PB TMD scheme selected " << endl ;
          allFlavuPDF(x,kt,mu, up, ubar, dn, dbar, strange, sbar, charm, cbar,  bottom, bbar, top, tbar, phot,  gluon ) ;        
          uval = up-ubar ;
          dval = dn-dbar ;
       }
       else if (  TMDgetScheme() == "Pavia TMD" ) { 
          cout << " TMDdensity:  Pavia TMD scheme selected " << endl ;
          PaviaTemp(x,kt,mu, up, ubar, dn, dbar, strange, sbar, charm, cbar,  bottom, bbar, top, tbar, phot,  gluon ) ;        
          uval = up-ubar ;
          dval = dn-dbar ;          
       }
       else { cout << " TMDdensity:  no valid scheme selected: EXIT_FAILURE " << endl; exit (EXIT_FAILURE); }
       
    }
    else if( NewFormat == 0) { 
       // cout << " checking on format " << NewFormat << endl;
       if ( iset >= 101000 && iset <=109999  ) {
         allFlavuPDF_old(x,kt,mu, up, ubar, dn, dbar, strange, sbar, charm, cbar,  bottom, bbar, top, tbar, phot,  gluon ) ;        
         uval = up-ubar ;
         dval = dn-dbar ;
         } 
       else if ( iset >= 800001 && iset <=800010  ) {
         allFlavuPDF_old(x,kt,mu, up, ubar, dn, dbar, strange, sbar, charm, cbar,  bottom, bbar, top, tbar, phot,  gluon ) ;        
         uval = up-ubar ;
         dval = dn-dbar ;
         }     
       else if ( iset >= 10000000  ) {
         allFlavuPDFn(x,kt,mu, up, ubar, dn, dbar, strange, sbar, charm, cbar,  bottom, bbar, top, tbar, phot,  gluon ) ;        
         uval = up-ubar ;
         dval = dn-dbar ;
         }     
       else if ( iset == 200001  ) {
         gbwuPDFlight(x,kt, gluon) ;
         } 
       else if ( iset == 200002  ) {
         gbwuPDFcharm(x,kt, gluon) ;
         } 
       else if ( iset == 210001  ) {
         gluon=blueml(&x,&kt,&mu) ;
         } 
       else if ( iset >= 300001 && iset <= 300003 ) {
         sbrsPDF(irep, imode, x, kt, mu, uval, dval, sea, charm) ;
         }   
       else if ( iset >= 400000 && iset <= 400004 ) {
         ksPDF(x, kt, mu, up, dn, sea, charm, bottom, gluon ) ;
         uval = up-sea ;
         dval = dn-sea ;
         }
       else if ( iset == 400101 ) {
         ksDLCPDF(x, kt, mu, up, ubar, dn, dbar, strange, sbar, charm, cbar,  bottom, bbar, gluon ) ;
         uval = up - ubar;
         dval = dn - dbar;
         }    
       else if ( iset >= 410000 && iset <= 410100 ) {
         ksBHKSPDF(x, kt, mu, up, ubar, dn, dbar, strange, sbar, charm, cbar,  bottom, bbar, gluon ) ;
         uval = up-ubar ;
         dval = dn-dbar ;
         }    
       else if ( iset >= 500001 && iset <= 500004 ) {
         tmd_ME(x,kt,mu, uval, dval, sea, charm, bottom, gluon ) ;
         }
    
       else {
         cout << " iset not defined " << iset << endl;
         } 
    }
    else { cout << " NewFormat not defined " << NewFormat << endl ; }
    

//    cout << " TMDpdf: x, xbar, kt, mu " << x << " " << xbar << " " << kt << " " << mu << endl;

}



